/*

  SmartClient Ajax RIA system
  Version SNAPSHOT_v15.0d_2025-05-02/LGPL Deployment (2025-05-02)

  Copyright 2000 and beyond Isomorphic Software, Inc. All rights reserved.
  "SmartClient" is a trademark of Isomorphic Software, Inc.

  LICENSE NOTICE
     INSTALLATION OR USE OF THIS SOFTWARE INDICATES YOUR ACCEPTANCE OF
     ISOMORPHIC SOFTWARE LICENSE TERMS. If you have received this file
     without an accompanying Isomorphic Software license file, please
     contact licensing@isomorphic.com for details. Unauthorized copying and
     use of this software is a violation of international copyright law.

  DEVELOPMENT ONLY - DO NOT DEPLOY
     This software is provided for evaluation, training, and development
     purposes only. It may include supplementary components that are not
     licensed for deployment. The separate DEPLOY package for this release
     contains SmartClient components that are licensed for deployment.

  PROPRIETARY & PROTECTED MATERIAL
     This software contains proprietary materials that are protected by
     contract and intellectual property law. You are expressly prohibited
     from attempting to reverse engineer this software or modify this
     software for human readability.

  CONTACT ISOMORPHIC
     For more information regarding license rights and restrictions, or to
     report possible license violations, please contact Isomorphic Software
     by email (licensing@isomorphic.com) or web (www.isomorphic.com).

*/
isc.defineClass("SchemaValidator", "Validator");

isc.SchemaValidator.addClassProperties({
    create : isc.Validator.getSuperClass().create,

    __schemaValidatorsToInstall: [],
    _installSchemaValidators : function () {
        var schemaValidatorsToInstall = this.__schemaValidatorsToInstall;
        for (var rv = schemaValidatorsToInstall.length; rv > 0; --rv) {
            var v = rv - 1,
                schemaValidator = schemaValidatorsToInstall.removeAt(v);
            schemaValidator.__install();
        }
    }
});

isc.SchemaValidator.addProperties({
    type: "custom",
    clientOnly: true,
    defaultErrorMessage: "Must conform to schema",

    //typeName: null,
    //schemaFieldName: null,

    init : function () {
        this.Super("init", arguments);

        this.__install();
    },

    __install : function () {
        if (!isc.isA.nonemptyString(this.typeName)) {
            this.logError("'typeName' is required to be a non-empty string.");
            return;
        }
        if (!isc.isA.nonemptyString(this.schemaFieldName)) {
            this.logError("'schemaFieldName' is required to be a non-empty string.");
            return;
        }

        var schemaDS = isc.DS.get(this.typeName);
        if (!schemaDS) {
            isc.SchemaValidator.__schemaValidatorsToInstall.push(this);
            return;
        }

        var schemaField = schemaDS.getUnderlyingField(this.schemaFieldName);
        if (!schemaField) {
            this.logError("The schema for '" + this.typeName + "' does not have a '" + this.schemaFieldName + "' field.");
            return;
        }

        var validators = schemaField.validators;
        if (!validators) validators = schemaField.validators = [];
        validators.push(this);
    },

    condition : function (schemaField, thisValidator, value, defaults, additionalContext) {
        // It's easy to forget to use `thisValidator` when wanting to refer to the validator
        // instance; so, for convenience, a SchemaValidator can implement checkValue(),
        // which is invoked with the proper `this`.
        return thisValidator.checkValue(value, defaults, additionalContext);
    },

    checkValue : function (value, defaults, additionalContext) {
        return true;
    },

    _getDefaultValueOrChildNode : function (property, defaults, additionalContext) {
        if (property in defaults) return defaults[property];
        var node = additionalContext.node;
        if (node) {
            var children = node.children;
            if (isc.isAn.Array(children)) {
                var c = children.findIndex("_parentProperty", property);
                if (c >= 0) {
                    
                    return children[c];
                }
            }
        }
    },

    _getDataSourceProperty : function (property, defaults, additionalContext) {
        var dataSource = this._getDefaultValueOrChildNode(property, defaults, additionalContext);
        if (dataSource) {
            if (isc.isA.String(dataSource)) {
                dataSource = isc.DS.get(dataSource);
            } else if (isc.isAn.Object(dataSource) && !isc.isA.DataSource(dataSource)) {
                var dsDefaults = dataSource._origDefaults || dataSource.defaults;
                dataSource = !dsDefaults ? null : isc.DS.get(dsDefaults.ID);
            }
        }
        return dataSource;
    }
});
